@@ -18,11 +18,12 @@ from sqlalchemy.orm import relationship, Mapped, mapped_column, DeclarativeBase
 
 from lecrapaud.db.session import get_db
 from lecrapaud.db.models.base import Base
-from lecrapaud.db.models.feature_selection import feature_selection_association
+from lecrapaud.db.models.feature_selection import (
+    lecrapaud_feature_selection_association,
+)
 
 
 class Feature(Base):
-    __tablename__ = "features"
 
     id = Column(BigInteger, primary_key=True, index=True, autoincrement=True)
     created_at = Column(
@@ -39,7 +40,7 @@ class Feature(Base):
 
     feature_selections = relationship(
         "FeatureSelection",
-        secondary=feature_selection_association,
+        secondary=lecrapaud_feature_selection_association,
         back_populates="features",
         lazy="selectin",
     )
