@@ -26,21 +26,24 @@ from collections.abc import Iterable
 
 from lecrapaud.db.session import get_db
 from lecrapaud.db.models.base import Base, with_db
+from lecrapaud.config import LECRAPAUD_TABLE_PREFIX
 
 # jointures
 lecrapaud_feature_selection_association = Table(
-    "lecrapaud_feature_selection_association",
+    f"{LECRAPAUD_TABLE_PREFIX}_feature_selection_association",
     Base.metadata,
     Column(
         "feature_selection_id",
         BigInteger,
-        ForeignKey("lecrapaud_feature_selections.id", ondelete="CASCADE"),
+        ForeignKey(
+            f"{LECRAPAUD_TABLE_PREFIX}_feature_selections.id", ondelete="CASCADE"
+        ),
         primary_key=True,
     ),
     Column(
         "feature_id",
         BigInteger,
-        ForeignKey("lecrapaud_features.id", ondelete="CASCADE"),
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_features.id", ondelete="CASCADE"),
         primary_key=True,
     ),
 )
@@ -62,12 +65,12 @@ class FeatureSelection(Base):
     best_features_path = Column(String(255))
     experiment_id = Column(
         BigInteger,
-        ForeignKey("lecrapaud_experiments.id", ondelete="CASCADE"),
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_experiments.id", ondelete="CASCADE"),
         nullable=False,
     )
     target_id = Column(
         BigInteger,
-        ForeignKey("lecrapaud_targets.id", ondelete="CASCADE"),
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_targets.id", ondelete="CASCADE"),
         nullable=False,
     )
 
