@@ -19,6 +19,7 @@ from sqlalchemy.orm import relationship, Mapped, mapped_column, DeclarativeBase
 
 from lecrapaud.db.session import get_db
 from lecrapaud.db.models.base import Base
+from lecrapaud.config import LECRAPAUD_TABLE_PREFIX
 
 
 class ModelSelection(Base):
@@ -34,18 +35,20 @@ class ModelSelection(Base):
         nullable=False,
     )
     best_model_params = Column(JSON)
+    best_thresholds = Column(JSON)
     best_model_path = Column(String(255))
     best_model_id = Column(
-        BigInteger, ForeignKey("lecrapaud_models.id", ondelete="CASCADE")
+        BigInteger,
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_models.id", ondelete="CASCADE"),
     )
     target_id = Column(
         BigInteger,
-        ForeignKey("lecrapaud_targets.id", ondelete="CASCADE"),
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_targets.id", ondelete="CASCADE"),
         nullable=False,
     )
     experiment_id = Column(
         BigInteger,
-        ForeignKey("lecrapaud_experiments.id", ondelete="CASCADE"),
+        ForeignKey(f"{LECRAPAUD_TABLE_PREFIX}_experiments.id", ondelete="CASCADE"),
         nullable=False,
     )
 
