@@ -2,6 +2,7 @@ import os
 from pathlib import Path
 
 import pandas as pd
+import joblib
 
 # Set up coverage file path
 os.environ["COVERAGE_FILE"] = str(Path(".coverage").resolve())
@@ -13,7 +14,7 @@ from lecrapaud.db.session import get_db
 
 
 def create_experiment(
-    data: pd.DataFrame,
+    data: pd.DataFrame | str,
     corr_threshold,
     percentile,
     max_features,
@@ -22,6 +23,10 @@ def create_experiment(
     experiment_name,
     **kwargs,
 ):
+    if isinstance(data, str):
+        path = f"{data}/data/full.pkl" if "data" not in data else f"{data}/full.pkl"
+        data = joblib.load(path)
+
     dates = {}
     if date_column:
         dates["start_date"] = pd.to_datetime(data[date_column].iat[0])
@@ -34,7 +39,9 @@ def create_experiment(
 
     with get_db() as db:
         all_targets = Target.get_all(db=db)
-        targets = [target for target in all_targets if target.name in data.columns]
+        targets = [
+            target for target in all_targets if target.name in data.columns.str.upper()
+        ]
         experiment_name = f"{experiment_name}_{groups["number_of_groups"] if group_column else 'ng'}_{corr_threshold}_{percentile}_{max_features}_{dates['start_date'].date() if date_column else 'nd'}_{dates['end_date'].date() if date_column else 'nd'}"
 
         experiment_dir = f"{tmp_dir}/{experiment_name}"
