@@ -327,11 +327,11 @@ class PreprocessFeature:
         self.data.columns = self.data.columns.str.upper()
 
         self.experiment = experiment
-        self.columns_pca = columns_pca
-        self.columns_onehot = columns_onehot
-        self.columns_binary = columns_binary
-        self.columns_ordinal = columns_ordinal
-        self.columns_frequency = columns_frequency
+        self.columns_pca = [col.upper() for col in columns_pca]
+        self.columns_onehot = [col.upper() for col in columns_onehot]
+        self.columns_binary = [col.upper() for col in columns_binary]
+        self.columns_ordinal = [col.upper() for col in columns_ordinal]
+        self.columns_frequency = [col.upper() for col in columns_frequency]
         self.target_numbers = target_numbers
         self.target_clf = target_clf
 
@@ -493,6 +493,13 @@ class PreprocessFeature:
         for name, data in zip(["train", "val", "test"], [train, val, test]):
             logger.info(f"{data.shape} {name} data")
 
+        Experiment.upsert(
+            match_fields=["id"],
+            id=self.experiment_id,
+            train_size=len(train),
+            val_size=len(val),
+            test_size=len(test),
+        )
         return (
             train.reset_index(drop=True),
             val.reset_index(drop=True),
@@ -811,14 +818,34 @@ def traditional_descriptive_analysis(df: pd.DataFrame, group_column: str | None
         # Shape
         results["Shape"] = f"{df.shape[0]} rows × {df.shape[1]} columns"
 
-        # Duplicated rows
-        results["Duplicated rows"] = int(df.duplicated().sum())
+        # Create a copy of the DataFrame to avoid modifying the original
+        df_check = df.copy()
 
-        # Duplicated columns
-        duplicated_cols = df.T[df.T.duplicated()].index.tolist()
-        results["Duplicated columns"] = (
-            ", ".join(duplicated_cols) if len(duplicated_cols) > 0 else "None"
-        )
+        # Convert numpy arrays to tuples for hashing
+        for col in df_check.columns:
+            if df_check[col].apply(lambda x: isinstance(x, np.ndarray)).any():
+                df_check[col] = df_check[col].apply(
+                    lambda x: tuple(x) if isinstance(x, np.ndarray) else x
+                )
+
+        # Duplicated rows
+        results["Duplicated rows"] = int(df_check.duplicated().sum())
+
+        # Check for duplicated columns
+        try:
+            # Try to find duplicated columns
+            duplicated_cols = []
+            cols = df_check.columns
+            for i, col1 in enumerate(cols):
+                for col2 in cols[i + 1 :]:
+                    if df_check[col1].equals(df_check[col2]):
+                        duplicated_cols.append(f"{col1} = {col2}")
+
+            results["Duplicated columns"] = (
+                ", ".join(duplicated_cols) if duplicated_cols else "None"
+            )
+        except Exception as e:
+            results["Duplicated columns"] = f"Could not check: {str(e)}"
 
         # Missing values
         missing = df.isnull().sum()
