@@ -47,14 +47,13 @@ from lecrapaud.db import (
     FeatureSelection,
     FeatureSelectionRank,
 )
-from lecrapaud.db.session import get_db
 from lecrapaud.search_space import all_models
 
 # Annoying Warnings
 warnings.filterwarnings("ignore", category=FutureWarning)
 
 
-def load_train_data(experiment_dir, target_number, target_type="regression"):
+def load_train_data(experiment_dir):
     data_dir = f"{experiment_dir}/data"
 
     logger.info("Loading data...")
@@ -90,11 +89,9 @@ class FeatureSelectionEngine:
         self.experiment_dir = self.experiment.path
         self.experiment_id = self.experiment.id
         self.data_dir = f"{self.experiment_dir}/data"
-        self.preprocessing_dir = f"{self.experiment_dir}/preprocessing"
-        self.fs_dir_target = (
-            f"{self.experiment_dir}/{f"TARGET_{self.target_number}"}/feature_selection"
-        )
-        os.makedirs(self.fs_dir_target, exist_ok=True)
+        self.target_dir = f"{self.experiment_dir}/TARGET_{self.target_number}"
+        self.feature_selection_dir = f"{self.target_dir}/feature_selection"
+        os.makedirs(self.feature_selection_dir, exist_ok=True)
 
     # Main feature selection function
     def run(
@@ -110,7 +107,6 @@ class FeatureSelectionEngine:
         """
         target_number = self.target_number
         target_type = self.target_type
-        fs_dir_target = self.fs_dir_target
 
         # Create the feature selection in db
         target = Target.find_by(name=f"TARGET_{target_number}")
@@ -123,6 +119,7 @@ class FeatureSelectionEngine:
             target_id=target.id,
             experiment_id=self.experiment_id,
         )
+        feature_map = {f.name: f.id for f in Feature.get_all(limit=20000)}
 
         if feature_selection.best_features_path and os.path.exists(
             feature_selection.best_features_path
@@ -133,11 +130,11 @@ class FeatureSelectionEngine:
         self.y = self.train[f"TARGET_{target_number}"]
 
         logger.info(f"Starting feature selection for TARGET_{target_number}...")
-        clean_directory(self.fs_dir_target)
+        clean_directory(self.feature_selection_dir)
 
-        # Let's start by removing extremly correlated features
-        # This is needed to reduce nb of feature but also for methods such as anova or chi2 that requires independent features
-        # TODO: we could also remove low variance features
+        # Let's start by removing very low variance feature and extremly correlated features
+        # This is needed to reduce nb of feature but also for methods such as anova or chi2 that requires independent, non constant, non full 0 features
+        self.X = self.remove_low_variance_columns()
         features_uncorrelated, features_correlated = self.remove_correlated_features(
             90, vizualize=False
         )
@@ -158,30 +155,37 @@ class FeatureSelectionEngine:
         self.X_categorical, self.X_numerical = get_features_by_types(self.X)
 
         if target_type == "classification" and self.X_categorical.shape[1] > 0:
-            feat_scores = self.select_categorical_features(
-                percentile=percentile, save_dir=fs_dir_target
-            )
-            with get_db() as db:
-                for row in feat_scores.itertuples(index=False):
-                    feature = Feature.find_by(name=row.features, db=db)
-                    FeatureSelectionRank.upsert(
-                        ["feature_selection_id", "feature_id", "method"],
-                        db=db,
-                        score=row.score,
-                        pvalue=row.pvalue,
-                        support=row.support,
-                        rank=row.rank,
-                        method=row.method,
-                        training_time=row.training_time,
-                        feature_selection_id=feature_selection.id,
-                        feature_id=feature.id,
-                    )
+            feat_scores = self.select_categorical_features(percentile=percentile)
+            rows = []
+            for row in feat_scores.itertuples(index=False):
+                feature_id = feature_map.get(row.features)
+
+                rows.append(
+                    {
+                        "feature_selection_id": feature_selection.id,
+                        "feature_id": feature_id,
+                        "method": row.method,
+                        "score": row.score,
+                        "pvalue": row.pvalue,
+                        "support": row.support,
+                        "rank": row.rank,
+                        "training_time": row.training_time,
+                    }
+                )
+
+            if len(rows) == 0:
+                logger.warning(
+                    f"No categorical features selected for TARGET_{target_number}"
+                )
+
+            FeatureSelectionRank.bulk_upsert(rows=rows)
+
             categorical_features_selected = feat_scores[feat_scores["support"]][
                 "features"
             ].values.tolist()
 
         results = []
-        params = {"percentile": percentile, "save_dir": fs_dir_target}
+        params = {"percentile": percentile}
         if single_process:
             results = [
                 self.select_feature_by_linear_correlation(**params),
@@ -243,30 +247,26 @@ class FeatureSelectionEngine:
 
         logger.info("Inserting feature selection results to db...")
         rows = []
-        with get_db() as db:
-            feature_map = {f.name: f.id for f in Feature.get_all(db=db, limit=20000)}
-            for row in feat_scores.itertuples(index=False):
-                feature_id = feature_map.get(row.features)
-                if not feature_id:
-                    continue  # or raise if feature must exist
+        for row in feat_scores.itertuples(index=False):
+            feature_id = feature_map.get(row.features)
 
-                rows.append(
-                    {
-                        "feature_selection_id": feature_selection.id,
-                        "feature_id": feature_id,
-                        "method": row.method,
-                        "score": row.score,
-                        "pvalue": None if pd.isna(row.pvalue) else row.pvalue,
-                        "support": row.support,
-                        "rank": row.rank,
-                        "training_time": row.training_time,
-                    }
-                )
+            rows.append(
+                {
+                    "feature_selection_id": feature_selection.id,
+                    "feature_id": feature_id,
+                    "method": row.method,
+                    "score": row.score,
+                    "pvalue": None if pd.isna(row.pvalue) else row.pvalue,
+                    "support": row.support,
+                    "rank": row.rank,
+                    "training_time": row.training_time,
+                }
+            )
 
-            if len(rows) == 0:
-                raise ValueError(f"No features selected for TARGET_{target_number}")
+        if len(rows) == 0:
+            logger.warning(f"No numerical features selected for TARGET_{target_number}")
 
-            FeatureSelectionRank.bulk_upsert(rows=rows, db=db)
+        FeatureSelectionRank.bulk_upsert(rows=rows)
 
         # Merge the results
         logger.info("Merging feature selection methods...")
@@ -288,7 +288,7 @@ class FeatureSelectionEngine:
         )
         logger.debug(features_selected_by_every_methods)
         pd.Series(features_selected_list).to_csv(
-            f"{fs_dir_target}/features_before_corr.csv",
+            f"{self.feature_selection_dir}/features_before_corr.csv",
             index=True,
             header=True,
             index_label="ID",
@@ -298,7 +298,7 @@ class FeatureSelectionEngine:
         self.X = self.X[features_selected_list]
         features, features_correlated = self.remove_correlated_features(corr_threshold)
         pd.Series(features).to_csv(
-            f"{fs_dir_target}/features_before_max.csv",
+            f"{self.feature_selection_dir}/features_before_max.csv",
             index=True,
             header=True,
             index_label="ID",
@@ -327,9 +327,7 @@ class FeatureSelectionEngine:
         )
 
         # save to path
-        best_features_path = Path(
-            f"{self.preprocessing_dir}/features_{target_number}.pkl"
-        ).resolve()
+        best_features_path = Path(f"{self.target_dir}/features.pkl").resolve()
         joblib.dump(features, best_features_path)
 
         # save in db
@@ -352,6 +350,32 @@ class FeatureSelectionEngine:
     # Remove correlation
     # ------------------
 
+    def remove_low_variance_columns(self, threshold: float = 1e-10) -> pd.DataFrame:
+        """
+        Removes columns with very low variance (including constant columns).
+
+        Parameters:
+            threshold (float): Minimum variance required to keep a column.
+                            Default is 1e-10 to eliminate near-constant features.
+
+        Returns:
+            pd.DataFrame: Cleaned DataFrame without low-variance columns.
+        """
+        X = self.X
+
+        low_var_cols = [
+            col
+            for col in X.columns
+            if pd.api.types.is_numeric_dtype(X[col])
+            and np.nanvar(X[col].values) < threshold
+        ]
+
+        if low_var_cols:
+            logger.info(f"🧹 Removed {len(low_var_cols)} low-variance columns:")
+            logger.info(low_var_cols)
+
+        return X.drop(columns=low_var_cols, errors="ignore")
+
     def remove_correlated_features(self, corr_threshold: int, vizualize: bool = False):
         X = self.X
         features = X.columns
@@ -397,7 +421,7 @@ class FeatureSelectionEngine:
     # Filter methods
     # ----------------
 
-    def select_categorical_features(self, percentile, save_dir: Optional[str] = None):
+    def select_categorical_features(self, percentile):
         X, y = self.X_categorical, self.y
 
         start = time.time()
@@ -420,15 +444,16 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/Chi2.csv", index=True, header=True, index_label="ID"
+            f"{self.feature_selection_dir}/Chi2.csv",
+            index=True,
+            header=True,
+            index_label="ID",
         )
 
         return feat_scores
 
     # Linear correlation (Person's R for regression and ANOVA for classification)
-    def select_feature_by_linear_correlation(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_linear_correlation(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -454,7 +479,7 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/{test_type}.csv",
+            f"{self.feature_selection_dir}/{test_type}.csv",
             index=True,
             header=True,
             index_label="ID",
@@ -463,9 +488,7 @@ class FeatureSelectionEngine:
         return feat_scores
 
     # Non-Linear correlation (Spearsman's R for regression and Kendall's Tau for classification)
-    def select_feature_by_nonlinear_correlation(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_nonlinear_correlation(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -511,7 +534,7 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/{test_type}.csv",
+            f"{self.feature_selection_dir}/{test_type}.csv",
             index=True,
             header=True,
             index_label="ID",
@@ -520,9 +543,7 @@ class FeatureSelectionEngine:
         return feat_scores
 
     # Mutual Information
-    def select_feature_by_mi(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_mi(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -549,7 +570,10 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/MI.csv", index=True, header=True, index_label="ID"
+            f"{self.feature_selection_dir}/MI.csv",
+            index=True,
+            header=True,
+            index_label="ID",
         )
 
         return feat_scores
@@ -558,9 +582,7 @@ class FeatureSelectionEngine:
     # ----------------
 
     # feature importance
-    def select_feature_by_feat_imp(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_feat_imp(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -602,7 +624,10 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/FI.csv", index=True, header=True, index_label="ID"
+            f"{self.feature_selection_dir}/FI.csv",
+            index=True,
+            header=True,
+            index_label="ID",
         )
 
         return feat_scores
@@ -611,9 +636,7 @@ class FeatureSelectionEngine:
     # ----------------
 
     # recursive feature elimination
-    def select_feature_by_rfe(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_rfe(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -655,15 +678,16 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/RFE.csv", index=True, header=True, index_label="ID"
+            f"{self.feature_selection_dir}/RFE.csv",
+            index=True,
+            header=True,
+            index_label="ID",
         )
 
         return feat_scores
 
     # SequentialFeatureSelector (loss based, possibility to do forwards or backwards selection or removal)
-    def select_feature_by_sfs(
-        self, percentile: int = 20, save_dir: Optional[str] = None
-    ):
+    def select_feature_by_sfs(self, percentile: int = 20):
         X, y, target_type = self.X_numerical, self.y, self.target_type
 
         start = time.time()
@@ -756,7 +780,10 @@ class FeatureSelectionEngine:
         )
 
         feat_scores.to_csv(
-            f"{save_dir}/SFS.csv", index=True, header=True, index_label="ID"
+            f"{self.feature_selection_dir}/SFS.csv",
+            index=True,
+            header=True,
+            index_label="ID",
         )
 
         return feat_scores
@@ -836,11 +863,13 @@ class PreprocessModel:
             train_scaled = None
             val_scaled = None
             test_scaled = None
+            scaler_x = None
 
         # save data
         joblib.dump(train_scaled, f"{self.data_dir}/train_scaled.pkl")
         joblib.dump(val_scaled, f"{self.data_dir}/val_scaled.pkl")
         joblib.dump(test_scaled, f"{self.data_dir}/test_scaled.pkl")
+        joblib.dump(scaler_x, f"{self.preprocessing_dir}/scaler_x.pkl")
 
         data = {
             "train": self.train,
@@ -849,7 +878,6 @@ class PreprocessModel:
             "train_scaled": train_scaled,
             "val_scaled": val_scaled,
             "test_scaled": test_scaled,
-            "scalers_y": scalers_y,
         }
 
         # reshape data for time series
@@ -876,10 +904,14 @@ class PreprocessModel:
         self.train = self.train[columns_to_keep]
 
         scaler_x = joblib.load(f"{self.preprocessing_dir}/scaler_x.pkl")
-        scaled_data = scaler_x.transform(self.train)  # TODO: utiliser scale_data
-        scaled_data = pd.DataFrame(
-            scaled_data, columns=self.train.columns, index=self.train.index
-        )
+
+        if scaler_x:
+            scaled_data = scaler_x.transform(self.train)
+            scaled_data = pd.DataFrame(
+                scaled_data, columns=self.train.columns, index=self.train.index
+            )
+        else:
+            scaled_data = self.train
 
         reshaped_data = None
         if (
@@ -925,7 +957,6 @@ class PreprocessModel:
             X_scaled = pd.DataFrame(
                 scaler_x.fit_transform(X), columns=list(X.columns), index=X.index
             )
-            joblib.dump(scaler_x, f"{self.preprocessing_dir}/scaler_x.pkl")
 
         # Determine which targets need to be scaled
         targets_numbers_to_scale = [
@@ -954,9 +985,8 @@ class PreprocessModel:
                     columns=y.columns,
                     index=y.index,
                 )
-                joblib.dump(
-                    scaler_y, f"{self.preprocessing_dir}/scaler_y_{target_number}.pkl"
-                )
+                target_dir = f"{self.experiment_dir}/TARGET_{target_number}"
+                joblib.dump(scaler_y, f"{target_dir}/scaler_y.pkl")
 
                 scalers_y[f"scaler_y_{target_number}"] = scaler_y
                 scaled_targets[target_number] = scaled_y
