@@ -114,6 +114,7 @@ class ModelEngine:
         self,
         model_name: str = None,
         target_type: str = None,
+        target_number: int = None,
         path: str = None,
         search_params: dict = {},
         create_model=None,
@@ -126,6 +127,7 @@ class ModelEngine:
         else:
             self.model_name = model_name
             self.target_type = target_type
+            self.target_number = target_number
 
         config = [
             config for config in all_models if config["model_name"] == self.model_name
@@ -145,11 +147,7 @@ class ModelEngine:
         self.log_dir = log_dir
 
         if self.path and self.need_scaling and self.target_type == "regression":
-            preprocessing_dir = Path(f"{self.path}/../preprocessing")
-            target_number = self.path.split("/")[-1].split("_")[-1]
-            self.scaler_y = joblib.load(
-                preprocessing_dir / f"scaler_y_{target_number}.pkl"
-            )
+            self.scaler_y = joblib.load(f"{self.path}/scaler_y.pkl")
         else:
             self.scaler_y = None
 
@@ -330,18 +328,16 @@ class ModelEngine:
         writer.close()
 
         if self.plot:
-            # Plot loss per epoch
-            train_loss = evals_result["train"][eval_metric]
-            val_loss = evals_result["val"][eval_metric]
-            logs = pd.DataFrame({"train": train_loss, "val": val_loss})
-
-            plt.figure(figsize=(14, 4))
-            plt.plot(logs.loc[:, "train"], lw=2, label="Training loss")
-            plt.plot(logs.loc[:, "val"], lw=2, label="Validation loss")
-            plt.xlabel("Epoch")
-            plt.ylabel("Loss")
-            plt.legend()
-            plt.show()
+            # Plot training progress
+            plot_training_progress(
+                logs={
+                    "train": evals_result["train"][eval_metric],
+                    "val": evals_result["val"][eval_metric],
+                },
+                model_name=self.model_name,
+                target_number=self.target_number,
+                title_suffix=f"Training Progress - {eval_metric}",
+            )
 
         self._model = model
 
@@ -469,16 +465,12 @@ class ModelEngine:
         # logger.info(pd.DataFrame(gradiant.epoch_gradient))
 
         if self.plot:
-            # Plot loss per epoch
-            logs = pd.DataFrame(history.history)
-
-            plt.figure(figsize=(14, 4))
-            plt.plot(logs.loc[:, "loss"], lw=2, label="Training loss")
-            plt.plot(logs.loc[:, "val_loss"], lw=2, label="Validation loss")
-            plt.xlabel("Epoch")
-            plt.ylabel("Loss")
-            plt.legend()
-            plt.show()
+            # Plot training progress using the utility function
+            plot_training_progress(
+                logs=history.history,
+                model_name=self.model_name,
+                target_number=self.target_number,
+            )
 
         self._model = model
 
@@ -571,51 +563,22 @@ class ModelEngine:
         if not self.path:
             raise ValueError("Path is not set, cannot load model")
 
-        training_target_dir = Path(self.path)
-
-        # Load threshold
-        scores_tracking = pd.read_csv(f"{training_target_dir}/scores_tracking.csv")
-        self.threshold = (
-            scores_tracking["THRESHOLD"].values[0]
-            if "THRESHOLD" in scores_tracking.columns
-            else None
-        )
-
-        # Search for files that contain '.best' or '.keras' in the name
-        best_files = list(training_target_dir.glob("*.best*")) + list(
-            training_target_dir.glob("*.keras*")
-        )
-        # If any files are found, try loading the first one (or process as needed)
-        if best_files:
-            file_path = best_files[
-                0
-            ]  # Assuming you want to open the first matching file
-            try:
-                # Attempt to load the file as a scikit-learn, XGBoost, or LightGBM model (Pickle format)
-                self._model = joblib.load(file_path)
-                logger.info(
-                    f"Loaded model {self._model.model_name} and threshold {self.threshold}"
-                )
-            except (pickle.UnpicklingError, EOFError):
-                # If it's not a pickle file, try loading it as a Keras model
-                try:
-                    # Attempt to load the file as a Keras model
-                    self._model = keras.models.load_model(file_path)
-                    logger.info(
-                        f"Loaded model {self._model.model_name} and threshold {self.threshold}"
-                    )
-                except Exception as e:
-                    raise FileNotFoundError(
-                        f"Model could not be loaded from path: {file_path}: {e}"
-                    )
-        else:
-            raise FileNotFoundError(
-                f"No files with '.best' or '.keras' found in the specified folder: {training_target_dir}"
-            )
+        self._model = load_model(self.path)
 
         self.model_name = self._model.model_name
         self.target_type = self._model.target_type
 
+        # Load threshold
+        self.threshold = (
+            joblib.load(f"{self.path}/thresholds.pkl")
+            if self.target_type == "classification"
+            else None
+        )
+
+        logger.info(
+            f"Loaded model {self._model.model_name} and threshold {self.threshold}"
+        )
+
 
 def trainable(
     params,
@@ -638,6 +601,7 @@ def trainable(
     model = ModelEngine(
         model_name=model_name,
         target_type=target_type,
+        target_number=target_number,
         create_model=create_model,
         plot=plot,
         log_dir=log_dir,
@@ -692,6 +656,9 @@ def trainable(
 
     score.update(evaluate(prediction, target_type, target_clf_thresholds))
 
+    metric = "RMSE" if target_type == "regression" else "LOGLOSS"
+    logger.info(f"{model.model_name} scores on validation set: {score[metric]:.4f}")
+
     if type_name == "hyperopts":
         session.report(metrics=score)
         return score
@@ -737,7 +704,7 @@ class ModelSelectionEngine:
         self.experiment_id = self.experiment.id
         self.data_dir = f"{self.experiment_dir}/data"
         self.preprocessing_dir = f"{self.experiment_dir}/preprocessing"
-        self.training_target_dir = f"{self.experiment_dir}/TARGET_{self.target_number}"
+        self.target_dir = f"{self.experiment_dir}/TARGET_{self.target_number}"
         self.metric = "RMSE" if self.target_type == "regression" else "LOGLOSS"
         self.features = self.experiment.get_features(self.target_number)
         self.all_features = self.experiment.get_all_features(
@@ -754,6 +721,7 @@ class ModelSelectionEngine:
         plot=True,
         clean_dir=False,  # TODO: This has been unused because now feature_selection is in the target directory
         preserve_model=True,
+        best_params=None,
     ):
         """
         Selects the best models based on a target variable, optionally performing hyperparameter optimization
@@ -826,7 +794,7 @@ class ModelSelectionEngine:
             if recurrent is False and config[self.target_type] is None:
                 continue  # for naive bayes models that cannot be used in regression
 
-            self.results_dir = f"{self.training_target_dir}/{model_name}"
+            self.results_dir = f"{self.target_dir}/{model_name}"
             if not os.path.exists(f"{self.results_dir}"):
                 os.makedirs(f"{self.results_dir}")
             elif preserve_model and contains_best(self.results_dir):
@@ -885,9 +853,10 @@ class ModelSelectionEngine:
                     x_test = test[self.features]
                     y_test = test[f"TARGET_{self.target_number}"].rename("TARGET")
 
-            log_dir = get_log_dir(self.training_target_dir, model_name)
+            log_dir = get_log_dir(self.target_dir, model_name)
             # instantiate model
             model = ModelEngine(
+                target_number=self.target_number,
                 model_name=model_name,
                 search_params=config["search_params"],
                 target_type=self.target_type,
@@ -899,29 +868,33 @@ class ModelSelectionEngine:
             start = time.time()
             # Tuning hyperparameters
             if perform_hyperopt:
-                best_params = self.hyperoptimize(x_train, y_train, x_val, y_val, model)
-
-                # save best params
-                best_params_file = f"{self.training_target_dir}/best_params.json"
-                try:
-                    with open(best_params_file, "r") as f:
-                        json_dict = json.load(f)
-                except FileNotFoundError:
-                    json_dict = {}
-
-                json_dict[model.model_name] = serialize_for_json(best_params)
-                with open(best_params_file, "w") as f:
-                    json.dump(json_dict, f, indent=4)
+                model_best_params = self.hyperoptimize(
+                    x_train, y_train, x_val, y_val, model
+                )
+            elif best_params:
+                model_best_params = best_params[model_name]
             else:
                 try:
-                    with open(f"{self.training_target_dir}/best_params.json") as f:
+                    with open(f"{self.target_dir}/best_params.json") as f:
                         json_dict = json.load(f)
-                        best_params = json_dict[model_name]
+                        model_best_params = json_dict[model_name]
                 except Exception:
                     raise FileNotFoundError(
-                        f"Could not find {model_name} in current data. Try to run an hyperoptimization by setting `perform_hyperopt` to true"
+                        f"Could not find {model_name} in current data. Try to run an hyperoptimization by setting `perform_hyperopt` to true, pass `best_params`"
                     )
 
+            # save best params
+            best_params_file = f"{self.target_dir}/best_params.json"
+            try:
+                with open(best_params_file, "r") as f:
+                    json_dict = json.load(f)
+            except FileNotFoundError:
+                json_dict = {}
+
+            json_dict[model.model_name] = serialize_for_json(model_best_params)
+            with open(best_params_file, "w") as f:
+                json.dump(json_dict, f, indent=4)
+
             # Perform cross-validation of the best model on k-folds of train + val set
             if perform_crossval:
                 x_train_val = pd.concat([x_train, x_val, x_test], axis=0)
@@ -932,16 +905,22 @@ class ModelSelectionEngine:
                 tscv = TimeSeriesSplit(n_splits=n_splits, test_size=test_size)
 
                 # Store the scores
-                cross_validation_scores = []
+                cv_scores = []
 
                 for i, (train_index, val_index) in enumerate(tscv.split(x_train_val)):
                     self.type_name = f"crossval_fold_{i}"
 
                     if self.time_series:
-                        date_series = train[self.date_column].copy()
+                        date_series = pd.concat(
+                            [
+                                train[self.date_column],
+                                val[self.date_column],
+                                test[self.date_column],
+                            ],
+                            axis=0,
+                        ).reset_index(drop=True)
 
-                        if need_scaling:
-                            date_series = date_series.map(pd.Timestamp.fromordinal)
+                        date_series = date_series.map(pd.Timestamp.fromordinal)
 
                         # Now you can use the actual train/val indices to extract ranges
                         train_start = date_series.iloc[train_index[0]]
@@ -960,8 +939,8 @@ class ModelSelectionEngine:
 
                     # Train the model and get the score
                     if recurrent:
-                        cross_validation_score, _, _ = self.train_model(
-                            params=best_params,
+                        cv_score, _, _ = self.train_model(
+                            params=model_best_params,
                             x_train=x_train_val[train_index],
                             y_train=y_train_val[train_index],
                             x_val=x_train_val[val_index],
@@ -969,8 +948,8 @@ class ModelSelectionEngine:
                             model=model,
                         )
                     else:
-                        cross_validation_score, _, _ = self.train_model(
-                            params=best_params,
+                        cv_score, _, _ = self.train_model(
+                            params=model_best_params,
                             x_train=x_train_val.iloc[train_index],
                             y_train=y_train_val.iloc[train_index],
                             x_val=x_train_val.iloc[val_index],
@@ -979,31 +958,39 @@ class ModelSelectionEngine:
                         )
 
                     # Append score to the list
-                    cross_validation_scores.append(cross_validation_score)
+                    cv_scores.append(cv_score)
 
-                # Calculate and log the mean score
-                cross_validation_mean_score = pd.DataFrame(cross_validation_scores)[
-                    self.metric
-                ].mean()
-                logger.info(
-                    f"Best model mean cross-validation score on entire experiment: {cross_validation_mean_score}"
-                )
+                # Calculate mean of all numerical metrics across all cross-validation folds
+                cv_scores_df = pd.DataFrame(cv_scores)
+                # Get mean of all numeric columns
+                cv_means = cv_scores_df.mean(numeric_only=True).to_dict()
+
+                logger.info(f"👉 {model.model_name} mean cv scores on full dataset:")
+                for metric, value in cv_means.items():
+                    logger.info(f"  {metric}: {value:.4f}")
 
                 # Retrain on entire training set, but keep score on cross-validation folds
-                best_score, best_model, best_pred = self.train_model(
-                    params=best_params,
+                # Get the test score using the best model
+                test_score, best_model, best_pred = self.train_model(
+                    params=model_best_params,
                     x_train=pd.concat([x_train, x_val], axis=0),
                     y_train=pd.concat([y_train, y_val], axis=0),
                     x_val=x_test,
                     y_val=y_test,
                     model=model,
                 )
-                best_score = cross_validation_mean_score
+
+                # Update all metrics with cross-validation means
+                for metric, value in cv_means.items():
+                    if metric in test_score:  # Only update existing metrics
+                        test_score[metric] = value
+                best_score = test_score
+                best_score["TYPE"] = "crossval"
             else:
-                # Evaluate on validation set
-                self.type_name = "validation"
+                # Evaluate on test set
+                self.type_name = "testset"
                 best_score, best_model, best_pred = self.train_model(
-                    params=best_params,
+                    params=model_best_params,
                     x_train=pd.concat([x_train, x_val], axis=0),
                     y_train=pd.concat([y_train, y_val], axis=0),
                     x_val=x_test,
@@ -1011,9 +998,12 @@ class ModelSelectionEngine:
                     model=model,
                 )
 
-                logger.info(f"Best model scores on test set: {best_score}")
+                logger.info(f"👉 {model.model_name} scores on test set:")
+                for metric, value in best_score.items():
+                    if isinstance(value, (int, float)):
+                        logger.info(f"  {metric}: {value:.4f}")
 
-            # Save validation predictions
+            # Save predictions
             best_pred.to_csv(
                 f"{self.results_dir}/prediction.csv",
                 index=True,
@@ -1027,8 +1017,8 @@ class ModelSelectionEngine:
             model_path = Path(model_path).resolve()
             best_score["MODEL_PATH"] = model_path
 
-            # Track scores
-            scores_tracking_path = f"{self.training_target_dir}/scores_tracking.csv"
+            # Save best scores
+            scores_tracking_path = f"{self.target_dir}/scores_tracking.csv"
             best_score_df = pd.DataFrame([best_score])
 
             if os.path.exists(scores_tracking_path):
@@ -1049,7 +1039,7 @@ class ModelSelectionEngine:
             # Save model training metadata
             stop = time.time()
             training_time = stop - start
-            model_training.best_params = best_params
+            model_training.best_params = model_best_params
             model_training.model_path = model_path
             model_training.training_time = training_time
             model_training.save()
@@ -1072,47 +1062,56 @@ class ModelSelectionEngine:
             logger.info(f"Model training finished in {training_time:.2f} seconds")
 
         # find best model type
-        scores_tracking_path = f"{self.training_target_dir}/scores_tracking.csv"
+        scores_tracking_path = f"{self.target_dir}/scores_tracking.csv"
         scores_tracking = pd.read_csv(scores_tracking_path)
         best_score_overall = scores_tracking.iloc[0, :]
         best_model_name = best_score_overall["MODEL_NAME"]
+        if self.target_type == "classification":
+            best_thresholds = best_score_overall["THRESHOLDS"]
+            joblib.dump(best_thresholds, f"{self.target_dir}/thresholds.pkl")
+        else:
+            best_thresholds = None
 
         # Remove any .best or .keras files
-        for file_path in glob.glob(
-            os.path.join(self.training_target_dir, "*.best")
-        ) + glob.glob(os.path.join(self.training_target_dir, "*.keras")):
+        for file_path in glob.glob(os.path.join(self.target_dir, "*.best")) + glob.glob(
+            os.path.join(self.target_dir, "*.keras")
+        ):
             os.remove(file_path)
         # Copy the best model in root training folder for this target
         best_model_path = Path(
-            f"{self.training_target_dir}/{os.path.basename(best_score_overall['MODEL_PATH'])}"
+            f"{self.target_dir}/{os.path.basename(best_score_overall['MODEL_PATH'])}"
         ).resolve()
         copy_any(
             best_score_overall["MODEL_PATH"],
             best_model_path,
         )
 
-        with open(f"{self.training_target_dir}/best_params.json", "r") as f:
+        with open(f"{self.target_dir}/best_params.json", "r") as f:
             best_model_params = json.load(f)[best_model_name]
 
-        # save model_selection results to db
+        # Save model_selection results to db
         model_selection = ModelSelection.get(model_selection.id)
         model_selection.best_model_id = Model.find_by(
             name=best_score_overall["MODEL_NAME"], type=self.target_type
         ).id
         model_selection.best_model_params = best_model_params
+        model_selection.best_thresholds = best_thresholds
         model_selection.best_model_path = best_model_path
         model_selection.save()
 
         logger.info(f"Best model overall is : {best_score_overall}")
 
+        # Consolidate best parameters from all targets into a single file
+        self.consolidate_best_params()
+
         best_model = joblib.load(best_model_path)
         return best_model
 
     def hyperoptimize(self, x_train, y_train, x_val, y_val, model: ModelEngine):
         self.type_name = "hyperopts"
 
-        def collect_error_logs(training_target_dir: int, storage_path: str):
-            output_error_file = f"{training_target_dir}/errors.log"
+        def collect_error_logs(target_dir: int, storage_path: str):
+            output_error_file = f"{target_dir}/errors.log"
 
             with open(output_error_file, "a") as outfile:
                 # Walk through the ray_results directory
@@ -1165,7 +1164,7 @@ class ModelSelectionEngine:
         try:
             results = tuner.fit()
 
-            best_result = results.get_best_result(self.metric, "max")
+            best_result = results.get_best_result(self.metric, "min")
             best_params = best_result.config
             best_score = best_result.metrics
 
@@ -1176,9 +1175,7 @@ class ModelSelectionEngine:
                 f"Markdown table with all trials :\n{results.get_dataframe().to_markdown()}"
             )
             # Collect errors in single file
-            collect_error_logs(
-                training_target_dir=self.training_target_dir, storage_path=storage_path
-            )
+            collect_error_logs(target_dir=self.target_dir, storage_path=storage_path)
 
         except Exception as e:
             raise Exception(e)
@@ -1208,11 +1205,53 @@ class ModelSelectionEngine:
             target_clf_thresholds=self.target_clf_thresholds,
         )
 
+    def consolidate_best_params(self):
+        """
+        Consolidate best parameters from all targets into a single JSON file in the preprocessing folder.
+        The output will be a dictionary with target numbers as keys and their best parameters as values.
+        """
+        # Initialize the consolidated parameters dictionary
+        all_best_params = {}
+
+        # Find all target directories
+        target_dirs = [
+            d for d in os.listdir(self.experiment_dir) if d.startswith("TARGET_")
+        ]
+
+        for target_dir in target_dirs:
+            target_number = target_dir.split("_")[1]
+            best_params_file = os.path.join(
+                self.experiment_dir, target_dir, "best_params.json"
+            )
+
+            # Check if best_params.json exists for this target
+            if os.path.exists(best_params_file):
+                try:
+                    with open(best_params_file, "r") as f:
+                        target_params = json.load(f)
+                        all_best_params[target_number] = target_params
+                except Exception as e:
+                    logger.warning(
+                        f"Error loading best params for {target_dir}: {str(e)}"
+                    )
+
+        # Save consolidated parameters to preprocessing folder
+        if all_best_params:
+            output_file = os.path.join(
+                self.preprocessing_dir, "all_targets_best_params.json"
+            )
+            os.makedirs(os.path.dirname(output_file), exist_ok=True)
+            with open(output_file, "w") as f:
+                json.dump(all_best_params, f, indent=4)
+            logger.info(f"Consolidated best parameters saved to {output_file}")
+
+        return all_best_params
+
 
 def evaluate(
     prediction: pd.DataFrame,
     target_type: str,
-    target_clf_thresholds: dict = {"precision": 0.80},
+    target_clf_thresholds: dict = None,
 ):
     """
     Function to evaluate model performance
@@ -1226,6 +1265,10 @@ def evaluate(
     y_true = prediction["TARGET"]
     y_pred = prediction["PRED"]
 
+    # Set default threshold if not provided
+    if target_clf_thresholds is None:
+        target_clf_thresholds = {"precision": 0.80}
+
     if target_type == "regression":
         # Main metrics
         score["RMSE"] = root_mean_squared_error(y_true, y_pred)
@@ -1321,67 +1364,207 @@ def evaluate(
 
 
 # utils
-def get_log_dir(training_target_dir: str, model_name="test_model"):
+def get_log_dir(target_dir: str, model_name="test_model"):
     """Generates a structured log directory path for TensorBoard."""
     timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M")
-    log_dir = (
-        Path(training_target_dir + "/tensorboard") / model_name / f"run_{timestamp}"
-    )
+    log_dir = Path(target_dir + "/tensorboard") / model_name / f"run_{timestamp}"
     log_dir.mkdir(parents=True, exist_ok=True)  # Create directories if they don't exist
     return str(log_dir)
 
 
-def print_scores(training_target_dir: str):
+def load_model(target_dir: str):
+    target_dir = Path(target_dir)
+    # Search for files that contain '.best' or '.keras' in the name
+    best_files = list(target_dir.glob("*.best*")) + list(target_dir.glob("*.keras*"))
+    # If any files are found, try loading the first one (or process as needed)
+    if best_files:
+        file_path = best_files[0]  # Assuming you want to open the first matching file
+        try:
+            # Attempt to load the file as a scikit-learn, XGBoost, or LightGBM model (Pickle format)
+            return joblib.load(file_path)
+        except (pickle.UnpicklingError, EOFError):
+            # If it's not a pickle file, try loading it as a Keras model
+            try:
+                # Attempt to load the file as a Keras model
+                return keras.models.load_model(file_path)
+            except Exception as e:
+                raise FileNotFoundError(
+                    f"Model could not be loaded from path: {file_path}: {e}"
+                )
+    else:
+        raise FileNotFoundError(
+            f"No files with '.best' or '.keras' found in the specified folder: {target_dir}"
+        )
+
+
+def plot_training_progress(
+    logs, model_name, target_number, title_suffix="Training Progress"
+):
     """
-    Monitor scores
+    Plot training and validation metrics during model training.
+
+    Args:
+        logs: DataFrame or dict containing training history
+        model_name: Name of the model being trained
+        target_number: Target number for the model
+        title_suffix: Optional suffix for the plot title
     """
-    scores_tracking = pd.read_csv(f"{training_target_dir}/scores_tracking.csv")
-    return scores_tracking
+    if isinstance(logs, dict):
+        logs = pd.DataFrame(logs)
+
+    plt.figure(figsize=(14, 4))
+
+    # Plot all metrics that exist in the logs
+    if "loss" in logs.columns:
+        plt.plot(logs["loss"], lw=2, label="Training loss")
+    if "val_loss" in logs.columns:
+        plt.plot(logs["val_loss"], lw=2, label="Validation loss")
+
+    # If no specific loss columns, plot all available metrics
+    if "loss" not in logs.columns and "val_loss" not in logs.columns and not logs.empty:
+        for col in logs.columns:
+            if col.startswith("val_"):
+                plt.plot(logs[col], "--", lw=2, label=f"Validation {col[4:]}")
+            else:
+                plt.plot(logs[col], lw=2, label=f"Training {col}")
+
+    plt.title(f"{model_name} - Target {target_number}\n{title_suffix}")
+    plt.xlabel("Epoch")
+    plt.ylabel("Metric Value")
+    plt.legend()
+    plt.grid(True, alpha=0.3)
+    plt.tight_layout()
+    plt.show()
 
 
 # plots
 def plot_evaluation_for_classification(prediction: dict):
     """
-    Args
-        prediction (pd.DataFrame): Should be a df with TARGET, PRED, 0, 1 columns for y_true value (TARGET), y_pred (PRED), and probabilities (for classification only : 0 and 1)
+    Plot evaluation metrics for classification tasks (both binary and multiclass).
+
+    Args:
+        prediction (pd.DataFrame): Should be a df with:
+            - TARGET: true labels
+            - PRED: predicted labels
+            - For binary: column '1' or 1 for positive class probabilities
+            - For multiclass: columns 2 onwards for class probabilities
     """
     y_true = prediction["TARGET"]
     y_pred = prediction["PRED"]
-    y_pred_proba = prediction[1] if 1 in prediction.columns else prediction["1"]
 
     # Plot confusion matrix
     plot_confusion_matrix(y_true, y_pred)
 
-    # Compute ROC curve and ROC area
-    fpr, tpr, thresholds = roc_curve(y_true, y_pred_proba)
-    roc_auc = auc(fpr, tpr)
+    # Determine if binary or multiclass
+    unique_labels = np.unique(y_true)
+    unique_labels = np.sort(unique_labels)
+    n_classes = len(unique_labels)
 
-    plt.figure(figsize=(8, 8))
-    plt.plot(
-        fpr, tpr, color="darkorange", lw=2, label="ROC curve (area = %0.2f)" % roc_auc
-    )
-    plt.plot([0, 1], [0, 1], color="navy", lw=2, linestyle="--")
-    plt.xlim([0.0, 1.0])
-    plt.ylim([0.0, 1.05])
-    plt.xlabel("False Positive Rate")
-    plt.ylabel("True Positive Rate")
-    plt.title("ROC Curve")
-    plt.legend(loc="lower right")
-    plt.show()
+    if n_classes <= 2:
+        # Binary classification
+        y_pred_proba = prediction[1] if 1 in prediction.columns else prediction["1"]
 
-    # Compute precision-recall curve
-    precision, recall, _ = precision_recall_curve(y_true, y_pred_proba)
-    average_precision = average_precision_score(y_true, y_pred_proba)
+        # Compute and plot ROC curve
+        fpr, tpr, _ = roc_curve(y_true, y_pred_proba)
+        roc_auc = auc(fpr, tpr)
 
-    plt.figure(figsize=(8, 8))
-    plt.step(recall, precision, color="b", alpha=0.2, where="post")
-    plt.fill_between(recall, precision, step="post", alpha=0.2, color="b")
-    plt.xlabel("Recall")
-    plt.ylabel("Precision")
-    plt.ylim([0.0, 1.05])
-    plt.xlim([0.0, 1.0])
-    plt.title("Precision-Recall Curve: AP={0:0.2f}".format(average_precision))
-    plt.show()
+        plt.figure(figsize=(8, 8))
+        plt.plot(
+            fpr,
+            tpr,
+            color="darkorange",
+            lw=2,
+            label=f"ROC curve (area = {roc_auc:0.2f})",
+        )
+        plt.plot([0, 1], [0, 1], color="navy", lw=2, linestyle="--")
+        plt.xlim([0.0, 1.0])
+        plt.ylim([0.0, 1.05])
+        plt.xlabel("False Positive Rate")
+        plt.ylabel("True Positive Rate")
+        plt.title("ROC Curve")
+        plt.legend(loc="lower right")
+        plt.show()
+
+        # Compute and plot precision-recall curve
+        precision, recall, _ = precision_recall_curve(y_true, y_pred_proba)
+        average_precision = average_precision_score(y_true, y_pred_proba)
+
+        plt.figure(figsize=(8, 8))
+        plt.step(recall, precision, color="b", alpha=0.2, where="post")
+        plt.fill_between(recall, precision, step="post", alpha=0.2, color="b")
+        plt.xlabel("Recall")
+        plt.ylabel("Precision")
+        plt.ylim([0.0, 1.05])
+        plt.xlim([0.0, 1.0])
+        plt.title(f"Precision-Recall Curve: AP={average_precision:0.2f}")
+        plt.show()
+
+    else:
+        # Multiclass classification
+        # Get class probabilities
+        pred_cols = [
+            col for col in prediction.columns if col not in ["ID", "TARGET", "PRED"]
+        ]
+        y_pred_proba = prediction[pred_cols].values
+
+        # Compute ROC curve and ROC area for each class
+        fpr = dict()
+        tpr = dict()
+        roc_auc = dict()
+
+        plt.figure(figsize=(10, 8))
+        colors = plt.cm.get_cmap("tab10")(np.linspace(0, 1, n_classes))
+
+        for i, (label, color) in enumerate(zip(unique_labels, colors)):
+            y_true_binary = (y_true == label).astype(int)
+            y_score = y_pred_proba[:, i]
+
+            fpr[i], tpr[i], _ = roc_curve(y_true_binary, y_score)
+            roc_auc[i] = auc(fpr[i], tpr[i])
+
+            plt.plot(
+                fpr[i],
+                tpr[i],
+                color=color,
+                lw=2,
+                label=f"Class {label} (area = {roc_auc[i]:0.2f})",
+            )
+
+        plt.plot([0, 1], [0, 1], "k--", lw=2)
+        plt.xlim([0.0, 1.0])
+        plt.ylim([0.0, 1.05])
+        plt.xlabel("False Positive Rate")
+        plt.ylabel("True Positive Rate")
+        plt.title("Multiclass ROC Curves (One-vs-Rest)")
+        plt.legend(loc="lower right")
+        plt.show()
+
+        # Compute PR curve for each class
+        plt.figure(figsize=(10, 8))
+
+        for i, (label, color) in enumerate(zip(unique_labels, colors)):
+            y_true_binary = (y_true == label).astype(int)
+            y_score = y_pred_proba[:, i]
+
+            precision, recall, _ = precision_recall_curve(y_true_binary, y_score)
+            average_precision = average_precision_score(y_true_binary, y_score)
+
+            plt.step(
+                recall,
+                precision,
+                color=color,
+                alpha=0.8,
+                where="post",
+                label=f"Class {label} (AP = {average_precision:0.2f})",
+            )
+
+        plt.xlabel("Recall")
+        plt.ylabel("Precision")
+        plt.ylim([0.0, 1.05])
+        plt.xlim([0.0, 1.0])
+        plt.title("Multiclass Precision-Recall Curves")
+        plt.legend(loc="lower left")
+        plt.show()
 
 
 def plot_confusion_matrix(y_true, y_pred):
@@ -1542,6 +1725,9 @@ def apply_thresholds(
                     pd.Series(np.where(exceeded, col, -np.inf), index=pred_proba.index)
                 )
 
+        # For each row:
+        # 1. If any threshold is exceeded, take the class with highest probability among exceeded
+        # 2. If no threshold is exceeded, take the class with highest probability overall
         if class_predictions:
             preds_df = pd.concat(class_predictions, axis=1)
             probs_df = pd.concat(class_probabilities, axis=1)
@@ -1631,44 +1817,13 @@ def plot_threshold(prediction, threshold, precision, recall):
 
 
 # OLD - to sort out
-def get_pred_distribution(training_target_dir: str, model_name="linear"):
-    """
-    Look at prediction distributions
-    """
-    prediction = pd.read_csv(
-        f"{training_target_dir}/{model_name}/prediction.csv",
-        index_col="ID",
-    )
-    prediction.describe()
 
 
-def plot_feature_importance(training_target_dir: str, model_name="linear"):
-    """
-    Monitor feature importance ranking to filter out unrelevant features
-    """
-    model = joblib.load(f"{training_target_dir}/{model_name}/{model_name}.best")
-    if hasattr(model, "feature_importances_"):
-        feature_importances_ = model.feature_importances_.flatten()
-    elif hasattr(model, "feature_importance"):
-        feature_importances_ = model.feature_importance.flatten()
-    elif hasattr(model, "coefs_"):
-        feature_importances_ = np.mean(model.coefs_[0], axis=1).flatten()
-    elif hasattr(model, "coef_"):
-        feature_importances_ = model.coef_.flatten()
-    else:
-        feature_importances_ = []
-
-    sns.barplot(
-        data=feature_importances_,
-        orient="h",
-    )
-
-
-def print_model_estimators(training_target_dir: str, model_name="linear"):
+def print_model_estimators(target_dir: str, model_name="linear"):
     """
     Look at a specific trained model
     """
-    model = joblib.load(f"{training_target_dir}/{model_name}/{model_name}.best")
+    model = joblib.load(f"{target_dir}/{model_name}/{model_name}.best")
     for i in range(0, 100):
         logger.info(model.estimators_[i].get_depth())
 
